MajorCourageTracker = {
	name = "MajorCourageTracker",

	-- Default settings
	defaults = {
		left = 1000,
		top = 500,
		maxRows = 6,
	},

	roleIcons = {
		[LFG_ROLE_DPS ] = "/esoui/art/lfg/lfg_icon_dps.dds",
		[LFG_ROLE_TANK] = "/esoui/art/lfg/lfg_icon_tank.dds",
		[LFG_ROLE_HEAL] = "/esoui/art/lfg/lfg_icon_healer.dds",
	},

	enabled = false,
	groupSize = 0,
	units = { },
	panels = { },

	debug = false,
};

function MajorCourageTracker.OnAddOnLoaded( eventCode, addonName )
	if (addonName ~= MajorCourageTracker.name) then return end

	EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_ADD_ON_LOADED);

	MajorCourageTracker.vars = ZO_SavedVars:NewAccountWide("MajorCourageTrackerSavedVariables", 1, nil, MajorCourageTracker.defaults, nil, "$InstallationWide");
	MajorCourageTracker.InitializeControls();

	EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_PLAYER_ACTIVATED, MajorCourageTracker.PlayerActivated);

	MajorCourageTracker.initializeSlashCommands()
end

function MajorCourageTracker.initializeSlashCommands()
    SLASH_COMMANDS["/bff"] = MajorCourageTracker.Reset
end

function MajorCourageTracker.PlayerActivated( eventCode, initial )
	-- Check wiki.esoui.com/AvA_Zone_Detection if we want to enable this for PvP

	if (MajorCourageTrackerData.zones[GetZoneId(GetUnitZoneIndex("player"))] or MajorCourageTracker.debug) then
		MajorCourageTracker.Reset();

		if (not MajorCourageTracker.enabled) then
			MajorCourageTracker.enabled = true;

			EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_JOINED, MajorCourageTracker.GroupUpdate);
			EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_LEFT, MajorCourageTracker.GroupUpdate);
			EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_ROLES_CHANGED, MajorCourageTracker.GroupMemberRolesChanged);
			EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_GROUP_SUPPORT_RANGE_UPDATE, MajorCourageTracker.GroupSupportRangeUpdate);
			EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_EFFECT_CHANGED, MajorCourageTracker.EffectChanged);

			SCENE_MANAGER:GetScene("hud"):AddFragment(MajorCourageTracker.fragment);
			SCENE_MANAGER:GetScene("hudui"):AddFragment(MajorCourageTracker.fragment);
		end
	else
		if (MajorCourageTracker.enabled) then
			MajorCourageTracker.enabled = false;

			EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_JOINED);
			EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_LEFT);
			EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_GROUP_MEMBER_ROLES_CHANGED);
			EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_GROUP_SUPPORT_RANGE_UPDATE);
			EVENT_MANAGER:UnregisterForEvent(MajorCourageTracker.name, EVENT_EFFECT_CHANGED);

			SCENE_MANAGER:GetScene("hud"):RemoveFragment(MajorCourageTracker.fragment);
			SCENE_MANAGER:GetScene("hudui"):RemoveFragment(MajorCourageTracker.fragment);
		end
	end
	zo_callLater(MajorCourageTracker.Reset, 500);
end

function MajorCourageTracker.GroupUpdate( eventCode )
	zo_callLater(MajorCourageTracker.Reset, 500);
end

function MajorCourageTracker.GroupMemberRolesChanged( eventCode, unitTag, dps, healer, tank )
	zo_callLater(function()
		if (MajorCourageTracker.units[unitTag]) then
			MajorCourageTracker.panels[MajorCourageTracker.units[unitTag].panelId].role:SetTexture(MajorCourageTracker.roleIcons[MajorCourageTracker.GetRole(unitTag)]);
		end
	end, 1500);
end

function MajorCourageTracker.GroupSupportRangeUpdate( eventCode, unitTag, status )
	if (MajorCourageTracker.units[unitTag]) then
		MajorCourageTracker.UpdateRange(MajorCourageTracker.units[unitTag].panelId, status);
	end
end

function MajorCourageTracker.EffectChanged( eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType, unitName, unitId, abilityId, sourceType )
	if (MajorCourageTrackerData.effects[abilityId] and MajorCourageTracker.units[unitTag]) then
		if (changeType ~= EFFECT_RESULT_FADED) then
			if (not MajorCourageTracker.units[unitTag].effects[abilityId]) then
				MajorCourageTracker.units[unitTag].count = MajorCourageTracker.units[unitTag].count + 1;
				MajorCourageTracker.UpdateStatus(unitTag);
			end
			MajorCourageTracker.units[unitTag].effects[abilityId] = endTime;
		else
			if (MajorCourageTracker.units[unitTag].effects[abilityId]) then
				MajorCourageTracker.units[unitTag].count = MajorCourageTracker.units[unitTag].count - 1;
				MajorCourageTracker.UpdateStatus(unitTag);
			end
			MajorCourageTracker.units[unitTag].effects[abilityId] = nil;
		end

		if (MajorCourageTracker.debug) then
			local entry = string.format("[%d] [%d/%d] %s - %d/%s/%d - %d", changeType, GetTimeStamp(), GetGameTimeMilliseconds(), GetUnitDisplayName(unitTag), abilityId, effectName, endTime, MajorCourageTracker.units[unitTag].count);
			table.insert(MajorCourageTracker.vars.debug, entry);
			if (MajorCourageTracker.units[unitTag].self) then
				CHAT_SYSTEM:AddMessage(entry);
			end
		end
	end
end

function MajorCourageTracker.OnMoveStop( )
	MajorCourageTracker.vars.left = MajorCourageTrackerFrame:GetLeft();
	MajorCourageTracker.vars.top = MajorCourageTrackerFrame:GetTop();
end

function MajorCourageTracker.InitializeControls( )
	local wm = GetWindowManager();

	for i = 1, GROUP_SIZE_MAX do
		local panel = wm:CreateControlFromVirtual("MajorCourageTrackerPanel" .. i, MajorCourageTrackerFrame, "MajorCourageTrackerPanel");

		MajorCourageTracker.panels[i] = {
			panel = panel,
			bg = panel:GetNamedChild("Backdrop"),
			name = panel:GetNamedChild("Name"),
			role = panel:GetNamedChild("Role"),
		};

		MajorCourageTracker.panels[i].bg:SetEdgeColor(0, 0, 0, 0);
	end

	MajorCourageTrackerFrame:ClearAnchors();
	MajorCourageTrackerFrame:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, MajorCourageTracker.vars.left, MajorCourageTracker.vars.top);

	MajorCourageTracker.fragment = ZO_HUDFadeSceneFragment:New(MajorCourageTrackerFrame);
end

function MajorCourageTracker.Reset( )
	if (MajorCourageTracker.debug) then
		CHAT_SYSTEM:AddMessage("[Healer BFF] Resetting");
	end

	MajorCourageTracker.groupSize = GetGroupSize();
	MajorCourageTracker.units = { };
	local width, height = MajorCourageTracker.panels[1].panel:GetDimensions();

	for i = 1, GROUP_SIZE_MAX do
		local soloPanel = i == 1 and MajorCourageTracker.groupSize == 0;

		if (i <= MajorCourageTracker.groupSize or soloPanel) then
			local unitTag = (soloPanel) and "player" or GetGroupUnitTagByIndex(i);

			MajorCourageTracker.units[unitTag] = {
				panelId = i,
				count = 0,
				effects = { },
				self = AreUnitsEqual("player", unitTag),
			};

			MajorCourageTracker.panels[i].name:SetText(GetUnitDisplayName(unitTag));
			MajorCourageTracker.panels[i].role:SetTexture(MajorCourageTracker.roleIcons[MajorCourageTracker.GetRole(unitTag)]);

			MajorCourageTracker.UpdateStatus(unitTag);
			MajorCourageTracker.UpdateRange(i, IsUnitInGroupSupportRange(unitTag));

			MajorCourageTracker.panels[i].panel:SetAnchor(TOPLEFT, MajorCourageTrackerFrame, TOPLEFT, width * math.floor((i - 1) / MajorCourageTracker.vars.maxRows), height * ((i - 1) % MajorCourageTracker.vars.maxRows));
			MajorCourageTracker.panels[i].panel:SetHidden(false);
		else
			MajorCourageTracker.panels[i].panel:SetAnchor(TOPLEFT, MajorCourageTrackerFrame, TOPLEFT, 0, 0);
			MajorCourageTracker.panels[i].panel:SetHidden(true);
		end
	end
end

function MajorCourageTracker.GetRole( unitTag )
	local role = GetGroupMemberAssignedRole(unitTag);

	if (role == LFG_ROLE_INVALID) then
		local _, isHealer, isTank = GetGroupMemberRoles(unitTag);

		if (isTank) then
			role = LFG_ROLE_TANK;
		elseif (isHealer) then
			role = LFG_ROLE_HEAL;
		else
			role = LFG_ROLE_DPS;
		end
	end

	return(role);
end

function MajorCourageTracker.UpdateStatus( unitTag )
	local bg = MajorCourageTracker.panels[MajorCourageTracker.units[unitTag].panelId].bg;

	if (MajorCourageTracker.units[unitTag].count < 1) then
		bg:SetCenterColor(0, 0, 0, 0.5);
	elseif (MajorCourageTracker.units[unitTag].self) then
		bg:SetCenterColor(1, 0.8, 0, 0.8);
	else
		bg:SetCenterColor(1, 0.8, 0, 0.6);
	end
end

function MajorCourageTracker.UpdateRange( panelId, status )
	if (status) then
		MajorCourageTracker.panels[panelId].panel:SetAlpha(1);
	else
		MajorCourageTracker.panels[panelId].panel:SetAlpha(0.5);
	end
end

function MajorCourageTracker.EnableDebug( )
	MajorCourageTracker.debug = true;

	MajorCourageTrackerData.effects[17906] = true; -- Crusher
	MajorCourageTrackerData.effects[17945] = true; -- Weakening
	MajorCourageTrackerData.effects[36245] = true; -- Concealed Weapon IV
	MajorCourageTrackerData.effects[37921] = true; -- Crippling Grasp IV
	MajorCourageTrackerData.effects[43083] = true; -- Consuming Trap IV
	MajorCourageTrackerData.effects[78046] = true; -- Ruthless Salvo Bleed (unused?)

	if (not MajorCourageTracker.vars.debug) then
		MajorCourageTracker.vars.debug = { };
	end

	MajorCourageTracker.PlayerActivated();
end

EVENT_MANAGER:RegisterForEvent(MajorCourageTracker.name, EVENT_ADD_ON_LOADED, MajorCourageTracker.OnAddOnLoaded);